/**
 *
 * Openutils web test utils (http://www.openmindlab.com/lab/products/testing4web.html)
 * Copyright (C)2008 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package net.sourceforge.openutils.testing4web;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.util.Hashtable;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.HttpUnitOptions;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.servletunit.ServletRunner;


/**
 * Base TestCase class for jsp/web tests. Developers should not use this class directly, but extends the junit or Testng
 * implementation, according to their preferences.
 * @author fgiust
 */
public abstract class BaseJspEmbeddedTest
{

    /**
     * Logger.
     */
    protected Logger log = LoggerFactory.getLogger(getClass());

    /**
     * HttpUnit ServletRunner.
     */
    protected ServletRunner runner;

    protected String getContext()
    {
        return "/context";
    }

    protected String getJspUrl(String jsp)
    {
        return "http://localhost" + getContext() + "/jsps/" + jsp;
    }

    /**
     * @see junit.framework.TestCase#setUp()
     */
    public void setUp() throws Exception
    {
        // need to pass a web.xml file to setup servletunit working directory
        ClassLoader classLoader = getClass().getClassLoader();
        URL webXmlUrl = classLoader.getResource("WEB-INF/web.xml");
        String path = URLDecoder.decode(webXmlUrl.getFile(), "UTF-8");

        HttpUnitOptions.setDefaultCharacterSet("utf-8");
        System.setProperty("file.encoding", "utf-8");

        // start servletRunner
        runner = new ServletRunner(new File(path), getContext());

        Hashtable<String, String> params = new Hashtable<String, String>();
        params.put("javaEncoding", "utf-8");
        runner.registerServlet("*.jsp", "org.apache.jasper.servlet.JspServlet", params);

        log.debug("ServletRunner setup OK");

    }

    /**
     * @see junit.framework.TestCase#tearDown()
     */
    public void tearDown() throws Exception
    {
        // shutdown servlet engine
        runner.shutDown();
    }

    protected void checkGetWithExpectedTxt(String jspname) throws Exception
    {
        WebRequest request = new GetMethodWebRequest(getJspUrl(jspname));

        WebResponse response = runner.getResponse(request);

        if (log.isDebugEnabled())
        {
            log.debug("RESPONSE: " + response.getText());
        }

        String responseText = response.getText();

        String expectedFileName = StringUtils.substringBeforeLast(jspname, ".") + ".txt";

        InputStream stream = getClass().getResourceAsStream("/jsps/" + expectedFileName);
        String expected = IOUtils.toString(stream);
        IOUtils.closeQuietly(stream);

        assertEquals(responseText, expected);
    }

    protected abstract void assertEquals(String expected, String actual);

}